﻿-- flagRSP
--
-- based on xTooltip by Author : Per 'Doxxan' Lönn Wege
--


local flagRSP_Debug = false;
local flagRSP_DebugSave = false;

--[[ flagRSP constants ]]--
local INITIALIZATION_DELAY = 0;

-- Localized strings for the tooltip use now Blizzard's texts from GlobalStrings.lua
local FlagRSP_Locale_AFK = gsub(MARKED_AFK_MESSAGE," %%s","");
local FlagRSP_Locale_NOTAFK = CLEARED_AFK;

--[[

flagRSP library functions.

]]--
FlagRSP = {
	isInitialized = false;

	displayBoxInterval = 10;
	rescaleBoxInterval = 120000;
	
	startTime = 0;

	newTickDisplayBox = 0;
	newTickRescaleBox = {
		FlagRSPInfoBox = 0,
		FRIENDLISTFrameInfoBox = 0
	},

	pName = "";
	rName = "";

	verMajor = 1;
	verMinor = 2;
	verPatch = 4;
	versionString = "1.2.4";

	notificationSeen = false;

	-- playerEnteredWorld = false;

	standby = false; -- TODO local variable with set/get
    
    selfcast = false;
};




-- Events ----------------------------------------

function FlagRSP:OnLoad(frame)
	frame:RegisterEvent("ADDON_LOADED"); -- 1 Fires when addon and saved variables are available ( after 3.0)
	frame:RegisterEvent("PLAYER_LOGIN"); -- 3 login / reload ui
    
	frame:RegisterEvent("UPDATE_MOUSEOVER_UNIT"); -- mouse over update
	frame:RegisterEvent("PLAYER_TARGET_CHANGED"); -- target update
	frame:RegisterEvent("MODIFIER_STATE_CHANGED"); -- listens to modifiers
    
	frame:RegisterEvent("ADDON_ACTION_FORBIDDEN");
	frame:RegisterEvent("ADDON_ACTION_BLOCKED");	
    
    frame:RegisterEvent("PLAYER_LOGOUT"); -- For saving variables
    
	-- frame:RegisterEvent("PLAYER_ENTERING_WORLD"); -- 4 zoning
	-- frame:RegisterEvent("VARIABLES_LOADED");  -- 1.1 Fires when ALL saved variables are loaded (until 3.0)
	-- frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	-- frame:RegisterEvent("CHAT_MSG_SYSTEM");	
		
    TooltipHandler:OnLoad( frame );
end

function FlagRSP:OnEvent(event, ...)
	local arg = {...};
	
    -- FIXME FlagRSP:OnEvent(event, ...) rework - much depraced?
    -- Game Events
	if event == "UPDATE_MOUSEOVER_UNIT" then
        self:OnEvent_UPDATEMOUSEOVERUNIT(arg);
	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
        -- print(debugstack());
		self:OnEvent_TARGETCHANGED(arg);
    elseif ( event == "MODIFIER_STATE_CHANGED" ) then
        self:OnEvent_MODIFIERSTATECHANGED(arg);
    
    -- Info Events
	elseif ( event == "ADDON_ACTION_FORBIDDEN" ) then
        -- print("ADDON_ACTION_FORBIDDEN"); -- DEBUG Remove
        self:OnEvent_ADDONACTIONFORBIDDEN(arg);
	elseif ( event == "ADDON_ACTION_BLOCKED" ) then
        -- print("ADDON_ACTION_BLOCKED"); -- DEBUG Remove
        self:OnEvent_ADDONACTIONBLOCKED(arg);
        
    -- Loading Process Events
    elseif (event == "ADDON_LOADED") then
        if ( arg[1] == "flagRSP" ) then
            self:InitVariables(arg);
        end
    elseif ( event == "PLAYER_LOGIN" ) then
		self:OnEvent_PLAYERLOGIN(arg);
        
    -- Shutdown Process Events
    elseif ( event == "PLAYER_LOGOUT" ) then
        self:OnEvent_LOGOUT(arg);
    end

end

function FlagRSP:InitVariables(arg)
    -- FlagRSP Init
    self.pName, self.rName = FlagRSPConfigure:Initialize();
    
    -- Saved Variables FlagRSP
    FlagRSPProtocol:Load(FlagRSPData)
    -- FlagRSP:Init();
    FlagRSPConfigure:InitializeSettings();
    -- FlagRSPConfigure:FixSavedVariables();
    
    -- Saved Variables Friendlist
    Friendlist:InitializeData();
    Friendlist:FixSavedVariables();
end

function FlagRSP:OnEvent_PLAYERLOGIN(arg)
    -- TODOI Implement standby recognition after loading variables
    -- if not self.playerEnteredWorld then
    -- self.pName, self.rName = FlagRSPConfigure:Initialize();
    
    -- Apply Settings
    FRIENDLIST_Update();
    FRIENDLISTMiniMap_OnLoad();

    local t = 0;
    while FlagRSP_Locale_OnLoad[t] ~= nil do
        FlagRSPStatic:PrintM(FlagRSPParser:ParseString(FlagRSP_Locale_OnLoad[t]),1.0, 1.0, 0.0);
        t = t + 1;
    end
   
    if FlagRSPConfigure:GetStandby() then
        FlagRSPStatic:PrintM(FlagRSP_Locale_StandBy);
        FlagRSPStatic:PrintM(FlagRSP_Locale_StandBy2);
        FlagRSPConfigure:SetStandby(true);
        self.standby = true;
    end
    FlagRSPProtocol:UpdateOwn();

    TooltipHandler:OnEvent_Init( arg )
    
    -- FlagRSPInfo:PurgeBoxList();
    FlagHandler:PurgeFlags();

    if ( FlagRSPConfigure:GetShowTips() ) then
        self:showDYKBox()
    end

    -- self.playerEnteredWorld = true;


    FlagRSPConfigure:SetButtonPosition(FlagRSPConfigure:GetButtonPosition());
    if FlagRSPConfigure:GetButtonState() == nil then
        FlagRSPConfigure:SetButtonState(true);
    end
    if FlagRSPConfigure:GetButtonState() then
        FRIENDLISTMinimapButton:Show();
    else
        FRIENDLISTMinimapButton:Hide();	
    end
    -- end
end

function FlagRSP:OnEvent_UPDATEMOUSEOVERUNIT(arg)
    -- local unitID = select(2, GameTooltip:GetUnit());
    -- print("UPDATE_MOUSEOVER_UNIT");

    Friendlist:UpdateEntry("mouseover", "unitID");

    local tooltipUnit = "mouseover";

    --Grab the name of who we've moused-over
    local tName, tUnit = GameTooltip:GetUnit();
    local name, realm = UnitName(tUnit or tooltipUnit);
    local guildName = GetGuildInfo(tUnit or tooltipUnit);
    name = tName or name;

    -- Request information protocol
    if UnitExists(tooltipUnit) and UnitPlayerControlled(tooltipUnit) then
        -- print("getting2 "..tUnit);
        local request = FlagRSPProtocol:Request(tooltipUnit, "ott");
        FlagRSPProtocol:Update(tooltipUnit, tooltipUnit, request);
    end

    --If they are on our list, update the tooltip and play a sound
    if (Friendlist:IsFriend(name)) then
        if (Friendlist:GetFriendstate(name) <= -10) then
            if (FlagRSPConfigure:GetSoundsSetting() == true) then
                if GetTime() > FriendlistFrame.soundTick then
                    PlaySound("igMiniMapZoomIn");
                    FriendlistFrame.soundTick = GetTime() + 0.5;
                end	
            end
        end
    elseif (Friendlist:IsFriend(guildName)) then
        if (Friendlist:GetFriendstate(guildName) <= -10) then
            if (FlagRSPConfigure:GetSoundsSetting() == true) then

                if GetTime() > FriendlistFrame.soundTick then
                    PlaySound("igMiniMapZoomIn");
                    FriendlistFrame.soundTick = GetTime() + 0.5;
                end
            end
        end
    end
    -- print("Event");
    self:UpdateTooltip();
end

function FlagRSP:OnEvent_MODIFIERSTATECHANGED(arg) -- FIXME Rework/Solve otherwise
    local key, state = arg[1], arg[2];
    
    if GetModifiedClick("SELFCAST") ~= "NONE" and strfind(key,GetModifiedClick("SELFCAST")) then
        FlagRSP.selfcast = not FlagRSP.selfcast;
    end
end

function FlagRSP:OnEvent_ADDONACTIONFORBIDDEN(arg)
    FlagRSP:ShowAddonBlocked(arg, "FlagRSP:OnEvent_ADDONACTIONFORBIDDEN", "ADDON_ACTION_FORBIDDEN");
    -- print("FlagRSP:OnEvent_ADDONACTIONFORBIDDEN("..tostring(arg)..")["..#(arg).."]: " .. tostring(arg[1]) .. " with " .. tostring(arg[2])); -- DEBUG remove
    -- if ( select(1, arg) ~= nil) then-- and select(2, arg) ~= nil ) then
        -- if ( select(1, arg) == "flagRSP" or select(1, arg) == "flagRSPLoader" or select(1, arg) == "FriendList" ) then
            -- local debugMsg;
            -- debugMsg = "Call stack was: " .. debugstack(3, 2, 3);
			-- self:PrintDebug("ADDON_ACTION_FORBIDDEN fired.  " .. select(1, arg) .. " attempted to call " .. select(2, arg) .. " when it was protected.");
			-- self:PrintDebug(debugMsg);
        -- end
    -- end
end

function FlagRSP:OnEvent_ADDONACTIONBLOCKED(arg)
    FlagRSP:ShowAddonBlocked(arg, "FlagRSP:OnEvent_ADDONACTIONBLOCKED", "ADDON_ACTION_BLOCKED");
    -- print("FlagRSP:OnEvent_ADDONACTIONBLOCKED("..tostring(arg)..")["..#(arg).."]: " .. tostring(arg[1]) .. " with " .. tostring(arg[2])); -- DEBUG remove
    -- if ( select(1, arg) ~= nil) then-- and select(2, arg) ~= nil ) then
        -- if ( select(1, arg) == "flagRSP" or select(1, arg) == "flagRSPLoader" or select(1, arg) == "FriendList" ) then
            -- local debugMsg;
            -- debugMsg = "Call stack was: " .. debugstack(3, 2, 3);
			-- self:PrintDebug("ADDON_ACTION_BLOCKED fired.  " .. select(1, arg) .. " attempted to call " .. select(2, arg) .. " when it was protected.");
			-- self:PrintDebug(debugMsg);
        -- end
    -- end
end

function FlagRSP:ShowAddonBlocked(arg, sig, name)
    if ( select(1, arg) ~= nil) then-- and select(2, arg) ~= nil ) then
        if ( select(1, arg) == "flagRSP" or select(1, arg) == "flagRSPLoader" or select(1, arg) == "FriendList" ) then
        
            print(sig .. "("..tostring(arg)..")["..#(arg).."]: " .. tostring(arg[1]) .. " with " .. tostring(arg[2])); -- DEBUG remove
            local debugMsg;
            debugMsg = "Full stack was: " .. debugstack();
            debugMsg = "Call stack was: " .. debugstack(3, 2, 3);
			self:PrintDebug(name.. " fired.  " .. select(1, arg) .. " attempted to call " .. select(2, arg) .. " when it was protected.");
			self:PrintDebug(debugMsg);
        end
    end
end

function FlagRSP:OnEvent_TARGETCHANGED(arg)

    -- flokru: I linked hiding of levels in Targetframe to level changing in tooltips.
    FlagRSP:UpdateTargetLevel( arg );

    -- Make sure target frame is back to original color
    local TargetFrameTexture = TargetFrame.borderTexture;
    TargetFrameTexture:SetVertexColor(1.0, 1.0, 1.0, TargetFrameTexture:GetAlpha());

    Friendlist:UpdateEntry("target","unitID");

    if UnitExists("target") and UnitPlayerControlled("target") then
        -- Request information from protocol
        local request = FlagRSPProtocol:Request("target", "oib");
        FlagRSPProtocol:Update("target", "target", request);
        
        FlagRSP:ModifyTargetFrame();
    end

    FlagRSPInfo.targetChanged = true; -- TODO: function for FlagRSPInfo: "TARGETCHANGED"
    FlagRSPInfo:displayBox("target");
end

function FlagRSP:OnEvent_LOGOUT(arg)
    FlagRSPProtocol:OnLogout();
end

--[[

FlagRSP:OnUpdate(elapsed)

-- OnUpdate Handler

]]--
function FlagRSP:OnUpdate(elapsed)
	-- self:checkAFK("");
	-- FlagRSPConfigure:InitializeSettings();
	
	-- Saves on calls to GetTime().
	local curTime = GetTime();
	
	if ( self.startTime == 0 ) then
		self.startTime = curTime;
	elseif ( not self.isInitialized and curTime > ( self.startTime + INITIALIZATION_DELAY ) ) then
		self.isInitialized = true;
		FlagRSPStatic:PrintM(FlagRSP_Locale_Initialized);
	end
	
	if ( curTime > self.newTickDisplayBox ) then
		FlagRSPInfo:displayBox("target");
		self.newTickDisplayBox = curTime + self.displayBoxInterval;
	end
    
    -- TooltipHandler:OnUpdate( elapsed );

	-- flokru: Targetframe updates on several events. I don't know which.
	--         So, we change level in targetframe here. Not an optimal solution.
	if ( FlagRSPConfigure:IsInitialized() and FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
		if UnitName("target") ~= nil then
			local targetLevel = UnitLevel("target");
			local targetLevelText = "";
			if targetLevel > 0 then
				local Diff = targetLevel - UnitLevel("player");
	
				if UnitClassification("target") == "elite" then
					Diff = Diff + 7;
				end
	
				if Diff <= -7 then targetLevelText = FlagRSP_Locale_TF_Epuny; end
				if Diff <= -5 and Diff > -7 then targetLevelText = FlagRSP_Locale_TF_Puny; end
				if Diff < -1 and Diff > -5 then targetLevelText = FlagRSP_Locale_TF_Weak; end
				if Diff >= -1 and Diff <= 1 then targetLevelText = FlagRSP_Locale_TF_Equal; end
				if Diff > 1 and Diff < 4 then targetLevelText = FlagRSP_Locale_TF_Strong; end
				if Diff >= 4 and Diff < 7 then targetLevelText = FlagRSP_Locale_TF_Vstrong; end
				if Diff >= 7 and Diff < 10 then targetLevelText = FlagRSP_Locale_TF_Estrong; end
				if Diff >= 10 then targetLevelText = FlagRSP_Locale_TF_Impossible; end
				if targetLevel == -1 then targetLevelText = FlagRSP_Locale_TF_Impossible; end

				TargetFrame.levelText:SetText(targetLevelText);
			end
		end
	end
end



--[[ === Settings of own flag === ]]--

--[[ Sets the char status and prints an info message
]]--
function FlagRSP:setCharStatus(status)
	local mesg;
	local changed = false;

	if status ~= FlagRSPConfigure:GetCharacterStatus() then
		if status == 1 then
			mesg = FlagRSP_Locale_OOCFlag;
		elseif status == 2 then
			mesg = FlagRSP_Locale_ICFlag;
		elseif status == 3 then
			mesg = FlagRSP_Locale_FFAICFlag;
		elseif status == 0 then
			mesg = FlagRSP_Locale_NoCFlag;
		elseif status == 4 then
			mesg = FlagRSP_Locale_STFlag;
		end
		
		FlagRSPConfigure:SetCharacterStatus(status);

		FlagRSPStatic:PrintM(mesg);
		changed = true;
	end

	return changed;
end

--[[ Sets the description and prints an info message
]]--
function FlagRSP:setDesc(desc)
	local changed = false;
	if desc ~= FlagRSPConfigure:GetDescription() then
		FlagRSPConfigure:SetRevision(FlagRSPConfigure:GetRevision() + 1);
		FlagRSPConfigure:SetDescription(desc);

		FlagRSPStatic:PrintM(FlagRSP_Locale_DescUpdate);

		changed = true;
	end

	return changed;
end

--[[ Sets the full name and prints info messages
    - Uses character name as default, if no name is given
]]--
function FlagRSP:setFullname(name)
	local changed = false;
	
	if ( string.len(name) > 150 ) then
		name = string.sub(name, 1, 150);
	end
	
	if ( name ~= FlagRSPConfigure:GetFullname() ) then
		FlagRSPConfigure:SetFullname(name);
		
		if ( name == " " or name == "" or name == UnitName("player") ) then
			-- Using default name
			local msg = string.gsub(FlagRSP_Locale_NoAltName, "%%s", UnitName("player"));
			FlagRSPStatic:PrintM(msg);
		else
			-- Using other, own name
			local msg = string.gsub(FlagRSP_Locale_NewAltName, "%%s", name);
			FlagRSPStatic:PrintM(msg);
		end
		
		changed = true;
	end
	
	return changed;
end

function FlagRSP:SetName(name) -- TODOSY Remove, shouln'd be necessary
	local changed = false;
	
	if ( string.len(name) > 50 ) then
		name = string.sub(name, 1, 50);
	end
	
	if ( name ~= FlagRSPConfigure:GetName() ) then
        local player,_ = UnitName("player");
		FlagRSPConfigure:SetName(name);
		
		if ( name == " " or name == "" or name == UnitName("player") ) then
			-- Print a message or something saying no first name.
			local msg = string.gsub(FlagRSP_Locale_NoAltName, "%%s", UnitName("player"));
			FlagRSPStatic:PrintM(msg);
		else
			-- Print another message that you have an alternate first name.
			local msg = string.gsub(FlagRSP_Locale_NewAltName, "%%s", name);
			FlagRSPStatic:PrintM(msg);
		end
		
		changed = true;
	end
	
	return changed;
end

function FlagRSP:setSurname(name) -- TODOSY Remove, shouln'd be necessary
	local changed = false;

	if string.len(name) > 50 then
		name = string.sub(name,1,50);
	end

	if name ~= FlagRSPConfigure:GetSurname() then
		FlagRSPConfigure:SetSurname(name);

		if name == " " or name == "" then
			FlagRSPStatic:PrintM(FlagRSP_Locale_NoSurname);
		else
            local player = UnitName("player");
			local msg = string.gsub(FlagRSP_Locale_NewSurname, "%%s", name);
			FlagRSPStatic:PrintM(msg);
		end

		changed = true;
	end

	return changed;
end

--[[ Sets a title and prints an info message
]]--
function FlagRSP:setTitle(title)
	local changed = false;

	if string.len(title) > 100 then
		title = string.sub(title,1,100);
	end
	if title ~= FlagRSPConfigure:GetTitle() then
		FlagRSPConfigure:SetTitle(title);

		if strtrim(title) == "" then
			FlagRSPStatic:PrintM(FlagRSP_Locale_NoTitle);
		else
			local msg = string.gsub(FlagRSP_Locale_NewTitle, "%%s", title);
			FlagRSPStatic:PrintM(msg);
		end

		changed = true;
	end

	return changed;
end


--[[ === Flag Config === ]]--

function FlagRSP:enableRPTags()
	self:setRPFlag(5);
	self:setNameDisp(true);
	FlagRSPConfigure:SetLevelDisplay("relative");

	-- flokru: Name and title notification, a bit quick and dirty.
	if FlagRSPConfigure:GetSurname() ~= nil then
		self:setSurname(FlagRSPConfigure:GetSurname());
	end
	if FlagRSPConfigure:GetTitle() ~= nil then
		self:setTitle(FlagRSPConfigure:GetTitle());
	end
end

function FlagRSP:disableRPTags()
	self:setRPFlag(0);
	self:setNameDisp(false);
	self:setLevelDisp("none");

	-- flokru: However, we should give a notification here as well, so that
	--         players notice that there are still tags set.
	-- flokru: Name and title notification, a bit quick and dirty.
	if FlagRSPConfigure:GetSurname() ~= nil then
		self:setSurname(FlagRSPConfigure:GetSurname());
	end
	if FlagRSPConfigure:GetTitle() ~= nil then
		self:setTitle(FlagRSPConfigure:GetTitle());
	end

end

--[[

FlagRSP:toggleRankDisp()

- Toggles displaying of PvP ranks.

]]--
function FlagRSP:toggleRankDisp()
	if not FlagRSPConfigure:GetPVPRanks() then
		self:setRankDisp(true);
	else
		self:setRankDisp(false);
	end
end

--[[

FlagRSP:toggleGuildDisp()

- Toggles displaying of guild names in tooltip.

]]--
function FlagRSP:toggleGuildDisp()
	if self:GetGuildDisp() == 0 then
		self:setGuildDisp(1);
	elseif self:GetGuildDisp() == 1 then
		self:setGuildDisp(-1);
	elseif self:GetGuildDisp() == -1 then
		self:setGuildDisp(0);
	end
end

--[[

FlagRSP:setRankDisp()

- Sets rank display.

]]--
function FlagRSP:setRankDisp(value)
	local mesg;

	if value == 0 then value = false; end
	if value == 1 then value = true; end

	if value ~= true and value ~= false then return; end

	if value then
		mesg = FlagRSP_Locale_ShowRanks;
        SetCVar("UnitNamePlayerPVPTitle",1);
		FlagRSPConfigure:SetPVPRanks(true);
	else
		mesg = FlagRSP_Locale_HideRanks;
        SetCVar("UnitNamePlayerPVPTitle",0);
		FlagRSPConfigure:SetPVPRanks(false);
	end

	FlagRSPStatic:PrintM(mesg);
end

--[[

FlagRSP:setGuildDisp()

- Sets guild display.

]]--
function FlagRSP:setGuildDisp(value)
	local mesg;

	if value == false then value = -1; end
	if value == true then value = 1; end

	if value ~= 1 and value ~= -1 and value ~= 0 then return; end

	if value == 1 then
		mesg = FlagRSP_Locale_ShowAllGuild;
		FlagRSPConfigure:SetGuildDisplay("all");
	elseif value == -1 then
		mesg = FlagRSP_Locale_HideGuild;
		FlagRSPConfigure:SetGuildDisplay("none");
	elseif value == 0 then
		mesg = FlagRSP_Locale_ShowKnownGuild;
		FlagRSPConfigure:SetGuildDisplay("known");
	end

	FlagRSPStatic:PrintM(mesg);
end

--[[

FlagRSP:GetGuildDisp()

- Gets guild display option.
-- Dirty hack to keep this compatible.
-- TODO: Fix whatever calls this to use "all", "known", "none" instead of numbers.

]]--
function FlagRSP:GetGuildDisp()
	local state;
	local number = -1; -- Defaults to display no guilds.
	
	state = FlagRSPConfigure:GetGuildDisplay();
	
	if ( state == "all" ) then
		number = 1;
	elseif ( state == "known" ) then
		number = 0;
	end
	
	return number;
end

--[[

FlagRSP:PlotStatus()

- Gives an overview over all settings.

]]--
function FlagRSP:PlotStatus()
	self:setRPFlag(FlagRSPConfigure:GetRPFlag());
	self:setCharStatus(FlagRSPConfigure:GetCharacterStatus());
	self:setNameDisp(self:GetNameDisp());
	self:setLevelDisp(self:GetLevelDisp());
	self:setRankDisp(FlagRSPConfigure:GetPVPRanks());
	self:setGuildDisp(self:GetGuildDisp());
	self:setSurname(FlagRSPConfigure:GetSurname());
	self:setTitle(FlagRSPConfigure:GetTitle());
end


--[[

FlagRSP:GetVersionNum(ver)

- Converts version string <ver> into numbers.

]]--
function FlagRSP:GetVersionNum(ver)
	local release, major, minor, dotPos;

    local addition = "";
    if ( string.find(ver,"%a") ) then
        ver, addition = strsplit(" ",ver,2);
    end
	dotPos = string.find(ver, "%.");
	if dotPos ~= nil then
		release = tonumber(string.sub(ver, 1, dotPos-1));
		ver = string.sub(ver, dotPos+1, string.len(ver));
	else
		release = -1;
	end

	dotPos = string.find(ver, "%.");
	if dotPos ~= nil then
		major = tonumber(string.sub(ver, 1, dotPos-1));
		ver = string.sub(ver, dotPos+1, string.len(ver));
	else
		major = -1;
	end

	minor = tonumber(string.sub(ver, 1, string.len(ver)));

	if minor == nil then
		minor = -1;
	end


	return release, major, minor;
end

--[[

FlagRSP:checkVersion(ver)

- Checks for a new version of flagRSP.

]]--
function FlagRSP:checkVersion(ver)

	local release, major, minor = self:GetVersionNum(ver);

	local newVersion = false;

	if release ~= nil and major ~= nil and minor ~= nil and release >= 0 and major >= 0 and minor >= 0 then
		if release > self.verMajor then
			newVersion = true;
		elseif major > self.verMinor and release == self.verMajor then
			newVersion = true;
		elseif minor > self.verPatch and major == self.verMinor and release == self.verMajor then
			newVersion = true;
		end
	end

	if newVersion then
		self:PrintDebug("New version available");
		self:showVersionNotification(release, major, minor);
	end
end

function FlagRSP:showVersionNotification(r, m, n)
	if not UnitAffectingCombat("player") and FlagRSPConfigure.GetVersionNotification and not self.notificationSeen then
		FlagRSPNewVersionNotification:SetWidth(450);
		FlagRSPNewVersionNotification:SetHeight(100);
		FlagRSPNewVersionNotification:Show();
		FlagRSPNewVersionNotification:SetBackdropColor(0,0,0,0.9);

		local text = FlagRSP_Locale_NewVersionText;

		text = string.gsub(text, "%%r", r)
		text = string.gsub(text, "%%m", m)
		text = string.gsub(text, "%%n", n)

		FlagRSPNewVersionNotificationTitleText:SetText(FlagRSP_Locale_NewVersionTitle);
		FlagRSPNewVersionNotificationText:SetText(text);
		FlagRSPNewVersionNotificationCheckButtonText:SetText(FlagRSP_Locale_NewVersionCheckButton);
		FlagRSPNewVersionNotificationText:SetWidth(410);

		FlagRSPNewVersionNotificationText:SetTextHeight(14);
		FlagRSPNewVersionNotification:SetScale(2);
		FlagRSPNewVersionNotification:SetScale(1);

		FlagRSPNewVersionNotificationCheckButton:SetPoint("BOTTOMLEFT", FlagRSPNewVersionNotification:GetName(), "BOTTOMLEFT", 20, FlagRSPNewVersionNotificationAccept:GetHeight()+20);

		FlagRSPNewVersionNotification:SetPoint("TOP", "UIParent", "TOP", 0, -150);
		FlagRSPNewVersionNotification:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPNewVersionNotificationText:GetHeight()-FlagRSPNewVersionNotificationAccept:GetHeight()-FlagRSPNewVersionNotificationCheckButton:GetHeight());
	

		self.notificationSeen = true;
	end
end

function FlagRSP:showEditBox(message)
	FlagRSPDescWarnBox.warning = "givenname";
	if ( message == nil ) then
		message = FlagRSP_Locale_DescWarnText;
		FlagRSPDescWarnBox.warning = "description";
    end
    FlagRSPDescWarnBox:SetWidth(550);
    FlagRSPDescWarnBox:SetHeight(100);
    FlagRSPDescWarnBox:Show();
    FlagRSPDescWarnBox:SetBackdropColor(0,0,0,0.9);

    FlagRSPDescWarnBoxTitleText:SetText(FlagRSP_Locale_DescWarnTitle);
    FlagRSPDescWarnBoxText:SetText(message);
    FlagRSPDescWarnBoxText:SetWidth(510);

    FlagRSPDescWarnBoxText:SetTextHeight(12);
    FlagRSPDescWarnBox:SetScale(2);
    FlagRSPDescWarnBox:SetScale(1);

    FlagRSPDescWarnBoxAccept:SetScript("OnClick", FlagRSPDescWarnBoxAccept_OnClick);

    FlagRSPDescWarnBox:SetPoint("TOP", "UIParent", "TOP", 0, -150);
    FlagRSPDescWarnBox:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPDescWarnBoxText:GetHeight()-FlagRSPDescWarnBoxAccept:GetHeight());

end

function FlagRSP:showDYKBox(nextTip)
	if nextTip == nil then
		nextTip = false;
	end

	FlagRSPDYKBox:SetWidth(550);
	FlagRSPDYKBox:SetHeight(100);
	FlagRSPDYKBox:Show();
	FlagRSPDYKBox:SetBackdropColor(0,0,0,0.9);

	if FlagRSPConfigure:GetShowTips() then
		FlagRSPDYKBoxCheckButton:SetChecked(0);
	else
		FlagRSPDYKBoxCheckButton:SetChecked(1);
	end

	local text = FlagRSP_Locale_TipText[math.random(1,table.getn(FlagRSP_Locale_TipText))];

	FlagRSPDYKBoxCheckButtonText:SetText(FlagRSP_Locale_DYKCheckText);

	FlagRSPDYKBoxTitleText:SetText(FlagRSP_Locale_DYKTitle);
	FlagRSPDYKBoxText:SetText(text);
	FlagRSPDYKBoxText:SetWidth(510);

	FlagRSPDYKBoxAccept:Hide();

	FlagRSPDYKBoxText:SetTextHeight(12);
	FlagRSPDYKBox:SetScale(2);
	FlagRSPDYKBox:SetScale(1);

	FlagRSPDYKBoxCheckButton:SetPoint("BOTTOMLEFT", "FlagRSPDYKBox", "BOTTOMLEFT", 20, FlagRSPDYKBoxClose:GetHeight()+20);

	FlagRSPDYKBox:SetPoint("TOP", "UIParent", "TOP", 0, -150);
	FlagRSPDYKBox:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPDYKBoxText:GetHeight()-FlagRSPDYKBoxAccept:GetHeight()- FlagRSPDYKBoxCheckButton:GetHeight());

end

function FlagRSP:showWelcomeBox(full)
	FlagRSPWelcomeBox:SetWidth(550);
	FlagRSPWelcomeBox:SetHeight(100);
	FlagRSPWelcomeBox:Show();
	FlagRSPWelcomeBox:SetBackdropColor(0,0,0,0.9);

	local text
	if full then
		text = FlagRSPParser:ParseString(FlagRSP_Locale_WelcomeHeader .. FlagRSP_Locale_WelcomeText);
	else
		text = FlagRSPParser:ParseString(FlagRSP_Locale_WelcomeHeader);
	end

	FlagRSPWelcomeBoxTitleText:SetText(FlagRSP_Locale_WelcomeTitle);
	FlagRSPWelcomeBoxText:SetText(text);
	FlagRSPWelcomeBoxText:SetWidth(510);

	FlagRSPWelcomeBoxText:SetTextHeight(12);
	FlagRSPWelcomeBox:SetScale(2);
	FlagRSPWelcomeBox:SetScale(1);

	FlagRSPWelcomeBox:SetPoint("TOP", "UIParent", "TOP", 0, -150);
	FlagRSPWelcomeBox:SetPoint("BOTTOM", "UIParent", "TOP", 0, -220-FlagRSPWelcomeBoxText:GetHeight()-FlagRSPWelcomeBoxAccept:GetHeight());
end

function FlagRSP:setNameDisp(value)
	local mesg;

	if value == 0 then value = false; end
	if value == 1 then value = true; end

	if value ~= true and value ~= false then return; end

	if value then
		mesg = FlagRSP_Locale_HideNames;
		FlagRSPConfigure:SetNameDisplay(true);
	else
		mesg = FlagRSP_Locale_UnhideNames;
		FlagRSPConfigure:SetNameDisplay(false);
	end

	FlagRSPStatic:PrintM(mesg);
end

function FlagRSP:GetNameDisp() -- TODO check
	return FlagRSPConfigure:GetNameDisplay();
end

function FlagRSP:setLevelDisp(value)
	local mesg;

	if ( value == "none" ) then
		mesg = FlagRSP_Locale_HideLevels;
		FlagRSPConfigure:SetLevelDisplay("relative");
	elseif ( value == "relative" ) then
		mesg = FLagRSP_Locale_ColoredLevels;
		FlagRSPConfigure:SetLevelDisplay("color");
	else
		mesg = FlagRSP_Locale_UnhideLevels;
		FlagRSPConfigure:SetLevelDisplay("none");
	end

	FlagRSPStatic:PrintM(mesg);
end

function FlagRSP:GetLevelDisp()
	return FlagRSPConfigure:GetLevelDisplay();
end

function FlagRSP:setRPFlag(RPType)
	local mesg;
	local changed = false;

	if RPType < 0 or RPType > 5 then return; end

	if RPType ~= FlagRSPConfigure:GetRPFlag() then
		if RPType == 0 then
			-- flokru: Now we save roleplaying behaviour before unsetting rp-flag.
			if FlagRSPConfigure:GetRPFlag() ~= 0 then
				FlagRSPConfigure:SetOldRPFlag(FlagRSPConfigure:GetRPFlag());
			end
			mesg = FlagRSP_Locale_NoRPFlagSet;
		elseif RPType == 1 then
			mesg = FlagRSP_Locale_NormalRPFlagSet;
		elseif RPType == 2 then
			mesg = FlagRSP_Locale_CasualFlagSet;
		elseif RPType == 3 then
			mesg = FlagRSP_Locale_FulltimeFlagSet;
		elseif RPType == 4 then
			mesg = FlagRSP_Locale_BeginnerFlagSet;
		elseif RPType == 5 then
			-- flokru: I love recursion :)
			if FlagRSPConfigure:GetOldRPFlag() ~= nil then
				self:setRPFlag(FlagRSPConfigure:GetOldRPFlag());
			else
				self:setRPFlag(1);
			end
			return;
		end


		FlagRSPStatic:PrintM(mesg);
		FlagRSPConfigure:SetRPFlag(RPType);

		changed = true;
	end

	return changed;
end

--[[

- FlagRSP:UpdateTooltip()

-- Updates the tooltip with the roleplaying data.

]]--
function FlagRSP:UpdateTooltip( reset )
    TooltipHandler:UpdateTooltip( reset );
end


function FlagRSP:PlotStats()
	FlagRSPStatic:PrintM("|CFFFFFF7FFLAGRSP STATISTICS:|r");

	FlagRSPStatic:PrintM("|CFFFFFF7FMemory statistics:|r");

    UpdateAddOnMemoryUsage()
	local memory, gcTreshold = gcinfo();
	FlagRSPStatic:PrintM("Memory used by complete interface (including addons): |CFFFFFF7F" .. math.ceil(memory*10)/10 .. "|r kB.");
	FlagRSPStatic:PrintM("Memory used by flagRSP: |CFFFFFF7F" .. math.ceil(GetAddOnMemoryUsage("flagRSP")) .. "|r kB.");
	-- FlagRSPStatic:PrintM("Memory used by flagRSP Profile: |CFFFFFF7F" .. math.ceil(GetAddOnMemoryUsage("flagRSP_profiles")) .. "|r kB.");
	-- FlagRSPStatic:PrintM("Memory used by flagRSP xtensionxtooltip: |CFFFFFF7F" .. math.ceil(GetAddOnMemoryUsage("flagRSP_xtensionxtooltip")) .. "|r kB.");
	
    local numEntries, flags = FlagHandler:GetStats();
	FlagRSPStatic:PrintM("|CFFFFFF7FRP flags used:|r");
	FlagRSPStatic:PrintM("Number of currently saved entries (i.e. characters) on all realms: |CFFFFFF7F" .. numEntries.. "|r.");
	FlagRSPStatic:PrintM("Number of flagRSP users without flag: |CFFFFFF7F" .. flags[0] .. "|r.");
	FlagRSPStatic:PrintM("Number of flagRSP users with standard flag: |CFFFFFF7F" .. flags[1] .. "|r.");
	FlagRSPStatic:PrintM("Number of flagRSP users with casual flag: |CFFFFFF7F" .. flags[2] .. "|r.");
	FlagRSPStatic:PrintM("Number of flagRSP users with fulltime flag: |CFFFFFF7F" .. flags[3] .. "|r.");
	FlagRSPStatic:PrintM("Number of flagRSP users with beginner flag: |CFFFFFF7F" .. flags[4] .. "|r.");

end




function FlagRSP:toggleHideNames()
	if not self:GetNameDisp() then
		self:setNameDisp(true);
	else
		self:setNameDisp(false);
	end
end

function FlagRSP:toggleLevelDisp()
	local display = self:GetLevelDisp();
	
	if ( display == "none" ) then
		self:setLevelDisp("relative");
	elseif ( display == "relative" ) then
		self:setLevelDisp("color");
	else
		self:setLevelDisp("none");
	end
end

function FlagRSPNewVersionNotificationCheckButton_OnClick()
	local checkState = FlagRSPNewVersionNotificationCheckButton:GetChecked();
	
	if checkState == nil then
		FlagRSPConfigure:SetVersionNotification(true)
	elseif checkState == 1 then
		FlagRSPConfigure:SetVersionNotification(false);
	end
end

function FlagRSPDYKBoxCheckButton_OnClick()
	local checkState = FlagRSPDYKBoxCheckButton:GetChecked();
	
	if checkState == nil then
		FlagRSPConfigure:SetShowTips(true);
	elseif checkState == 1 then
		FlagRSPConfigure:SetShowTips(false);
	end
end

function FlagRSPDescWarnBoxAccept_OnClick()
	FlagRSPDescWarnBox:Hide();

	if ( FlagRSPDescWarnBox.warning == "description" ) then
		FlagRSPData.descBoxWarningSeen = true;
	else
		FlagRSPData.nameBoxWarningSeen = true;
	end
end

function FlagRSPWelcomeBoxAccept_OnClick()
	FlagRSPWelcomeBox:Hide();

	if FlagRSPConfigure:GetShowTips() then
		FlagRSP:showDYKBox();
	end
end

function FlagRSP:CheckVersionPlugins(versionTable)
    if ( versionTable ~= nil ) then
        if ( versionTable.release < FlagRSP.verMajor ) then
            return false;
        elseif ( versionTable.major < FlagRSP.verMinor ) then
            return false;
        elseif ( versiontable.minor < FlagRSP.verPatch ) then
            return false;
        end
        return true;
    else
        return false;
    end
end

function FlagRSP:setStandby( standby ) -- TODOI implement standby function
    -- local standby = false;
    -- -- OnLoad behaviour:
    -- -- If backup is not set in the saved variable: no standby
    -- if ( FlagRSPData.standby == nil ) then
        -- standby = false;
    -- -- else: standby
    -- else
        -- standby = true;
    -- end

    -- if standby, flagrsp should not make itself visible, background is working

    -- Standby active
        -- Save information on tooltip, RPflag, Charflag, leveldisplay
        -- Hide InfoBox and InfoBoxButton
        -- Change level display to default
        -- Block Flag request
    -- Standby inactive
        -- Load information on tooltip, RPFlag, Charflag, leveldisplay
        -- Unhide InfoBox and InfoBoxButton
        -- Change level display to the setting before
        -- Unblock Flag request
end


function FlagRSP:UpdateTargetLevel( arg )
    if ( not FlagRSPConfigure:IsInitialized() ) then
        return;
    end
    
    if FlagRSPConfigure:GetLevelDisplay() == "relative" or FlagRSPConfigure:GetLevelDisplay() == "color" then
        local targetLevel = UnitLevel("target");
        
        if targetLevel > 0 then
            Diff = targetLevel - UnitLevel("player");
            
            -- Change color
            if UnitClassification("target") == "elite" then
                Diff = Diff + 5;
            end

            local r,g,b;
            if ( Diff >= 7 ) then
                r = QuestDifficultyColors["impossible"].r;
                g = QuestDifficultyColors["impossible"].g;
                b = QuestDifficultyColors["impossible"].b;
            elseif ( Diff >= 4 ) then
                r = QuestDifficultyColors["verydifficult"].r;
                g = QuestDifficultyColors["verydifficult"].g;
                b = QuestDifficultyColors["verydifficult"].b;
            elseif ( Diff >= -1 ) then
                r = QuestDifficultyColors["difficult"].r;
                g = QuestDifficultyColors["difficult"].g;
                b = QuestDifficultyColors["difficult"].b;
            elseif ( -Diff <= GetQuestGreenRange() ) then
                r = QuestDifficultyColors["standard"].r;
                g = QuestDifficultyColors["standard"].g;
                b = QuestDifficultyColors["standard"].b;
            else
                r = QuestDifficultyColors["trivial"].r;
                g = QuestDifficultyColors["trivial"].g;
                b = QuestDifficultyColors["trivial"].b;
            end

            TargetFrame.levelText:SetTextColor(r,g,b);
            
            -- Change text
            local targetLevelText = "";
            if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
                if ( targetLevel == -1 or Diff >= 10 ) then
                    targetLevelText = FlagRSP_Locale_TF_Impossible;
                elseif ( Diff >= 7 ) then
                    targetLevelText = FlagRSP_Locale_TF_Estrong;
                elseif ( Diff >= 4 ) then
                    targetLevelText = FlagRSP_Locale_TF_Vstrong;
                elseif ( Diff >= 2 ) then
                    targetLevelText = FlagRSP_Locale_TF_Strong;
                elseif ( Diff >= -1 ) then
                    targetLevelText = FlagRSP_Locale_TF_Equal;
                elseif ( Diff >= -GetQuestGreenRange()) then
                    targetLevelText = FlagRSP_Locale_TF_Weak;
                elseif ( Diff >= (-4 -GetQuestGreenRange())) then
                    targetLevelText = FlagRSP_Locale_TF_Puny;
                else
                    targetLevelText = FlagRSP_Locale_TF_Epuny;
                end
                
                TargetFrame.levelText:SetText(targetLevelText);
            end
        end
    end
end

function FlagRSP:ModifyTargetFrame()
    local TargetFrameTexture = TargetFrame.borderTexture;
    local targetName, targetRealm = UnitName("target");
    local guildName = GetGuildInfo("target");
    local r, g, b, t;

    local playerID = FlagRSP:BuildPlayerID(targetName, targetRealm);
    
    if (Friendlist:IsFriend(playerID)) then
        r,g,b,t = Friendlist:GetFriendstateInfo(Friendlist:GetFriendstate(playerID));
        TargetFrameTexture:SetVertexColor(r, g, b, TargetFrameTexture:GetAlpha());
        TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");
    elseif (Friendlist:IsFriend(guildName)) then
        r,g,b,t = Friendlist:GetFriendstateGuildInfo(Friendlist:GetFriendstate(guildName));
        TargetFrameTexture:SetVertexColor(r, g, b, TargetFrameTexture:GetAlpha());
        TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame");
    end
end


function FlagRSP:BuildPlayerID(name, realm)
    local playerID = name;
    
    realm = FlagHandler:GetRealm( realm )
    if ( realm ~= nil and realm ~= "" and realm ~= FlagHandler:GetRealm( FlagRSP.rName ) ) then
        playerID = playerID .. "-" .. realm;
    end
    
    return playerID;
end
--[[ === Help functions === ]]--

--[[

FlagRSP:PrintDebug(msg)

- Gives a debug message msg.

]]--
function FlagRSP:PrintDebug(msg)
	if flagRSP_Debug and msg ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage("|CFF" .. FlagRSPStatic:GetHex(1.0, 1.0, 0.5) .. "<flagRSP> DEBUG:|r " .. msg,1,1,1);
        -- ChatFrame7
	end
end
