﻿--[[ File: Friend List Lua file
     borrowed lots of code from Asher (Warsong) and his original KOS List .)    			
     ////////////////////////////////////////////////////////////////////
     18-Mar-05  code hijacked and messed up for FriendList - alpha release
     xoxo Zmrzlina

]]


-- Constant variables
-- FriendlistHandler.FL_TYPE = "char";
-- FriendlistHandler.FL_TYPE = "guild";
-- FriendlistHandler.FL_FRIENDSTATE_ENEMY = -10;
-- FriendlistHandler.FL_FRIENDSTATE_KNOWN = 0;
-- FriendlistHandler.FL_FRIENDSTATE_FRIEND = 10;

local rev = 1; -- Version of FlagRSP Friendlist for compability checks
-- Currently used fields:
    -- type, class, surname, name, race, title, guild, notes, level, friendstate, rank
    -- "index" is updated automatically by the friendlist
-- Depraced fields: altName
-- local supportedTags = {
    -- "type", "class", "surname", "name", "race", "title", "guild", "notes", "level", "friendstate", "rank"
-- }

Friendlist = {
	initialized = nil;
	settingsLoaded = nil;
	playerName = nil;
	realmName = nil;
	originalName = nil; -- Compatibility.
	onlineBufferList = {};
    intervallIsOnline = 10;
};


--[[

Friendlist:UpdateEntry(name,type)

-- Update entry for <name>. <type> determines how to update.

]]--
function Friendlist:UpdateEntry(name, type)

	if type == "flagRSP" then
		if ( FlagHandler:GetFlag("Name", name) ~= nil and FlagHandler:GetFlag("Name", name) ~= name ) then
			self:SetAltName(name, FlagHandler:GetFlag("Name", name));
		end
        self:SetFullname(name, TooltipHandler:GetFullname(name));
		self:SetSurname(name,FriendlistFrame:GetSurname(name));
		self:SetTitle(name,TooltipHandler:GetRPTitle(name));
	elseif type == "unitID" then
		local unitName = UnitName(name);
		self:SetClass(unitName, UnitClass(name));
		self:SetGuild(unitName, GetGuildInfo(name));
		self:SetLevel(unitName, UnitLevel(name));
		self:SetRace(unitName, UnitRace(name));
		if ( UnitPVPName(name) ~= nil and unitName ~= nil and UnitIsPlayer(name) ~= nil ) then
			self:SetRank(unitName, string.sub(UnitPVPName(name), 1, string.find(UnitPVPName(name), unitName)-1));
		end
	elseif type == "flist" then
		for i=1, GetNumFriends() do
			local name, level, class, area, connected = GetFriendInfo(i);

			if level ~= 0 then
				self:SetLevel(name, level);
				self:SetClass(name, class);
			end
		end
	end
end




--[[

Friendlist:GetFriendstateInfo(friendstate)

- Get info (r,g,b,text) for friend status friendstate.

]]--
function Friendlist:GetFriendstateInfo(friendstate)
	local r, g, b, text;

	text = FRIENDLIST_LOCALE_FRIENDSTATE_TEXT[friendstate];
	r=1; g=1; b=1;

	if friendstate == -10 then
		r=1; g=0; b=0;
	elseif friendstate == 10 then
		r=0; g=1; b=0;
	elseif friendstate == 0 then
		r=1; g=1; b=0.4;
	else
		r=1; g=1; b=0.4;
	end

	if text == nil then text = ""; end

	return r, g, b, text;
end


--[[

Friendlist:GetFriendstateGuildInfo(friendstate)

- Get info (r,g,b,text) for guild friend status friendstate.

]]--
function Friendlist:GetFriendstateGuildInfo(friendstate)
	local r, g, b, text;

	text = FRIENDLIST_LOCALE_GUILDFRIENDSTATE_TEXT[friendstate];
	r=1; g=1; b=0.6;

	if friendstate == -10 then
		r=1; g=0.33; b=0.33;
	elseif friendstate == 10 then
		r=0.33; g=1; b=0.33;
	end

	return r, g, b, text;
end


--[[

Friendlist:GetFriendstateText(name)

- Get player names friend status text.

]]--
function Friendlist:GetFriendstateText(name)
	local friendstate = self:GetFriendstate(name);
	local text = "";

	if friendstate ~= nil then
		if self:GetType(name) ~= "guild" then
			local r, g, b, t = self:GetFriendstateInfo(friendstate);
			if t ~= nil and t ~= "" then
				text = t;
			end
		else
			local r, g, b, t = self:GetFriendstateGuildInfo(friendstate);
			if t ~= nil and t ~= "" then
				text = t;
			end
		end
	end

	return text;
end


--[[

Friendlist:GetFriendstateColour(name)

- Get player names friend status text colour code.

]]--
function Friendlist:GetFriendstateColour(name)
	local friendstate = self:GetFriendstate(name);
	local text = "";

	if friendstate ~= nil then
		if self:GetType(name) ~= "guild" then
			local r, g, b, t = self:GetFriendstateInfo(friendstate);
			text = FlagRSPStatic:GetHex(r, g, b);
		else
			local r,g,b,t = self:GetFriendstateGuildInfo(friendstate);
			text = FlagRSPStatic:GetHex(r, g, b);
		end
	end

	return text;
end


--[[

Friendlist:checkOnline(name)

-- checks if player is online and puts information into an online buffer.

]]--
function Friendlist:checkOnline(name)
	local isOnline = 0;

    if ( name == nil ) then
        return 0;
    end

    -- Blizzard's Friendlist Online Check
	for i=1, GetNumFriends() do
		local n, _, _, _, connected = GetFriendInfo(i);
		if n == name then
			if connected then
				isOnline = 1;
			else
				isOnline = -1;
			end
		end
	end

    -- Guild rooster Online Check
    if ( isOnline == 0 ) then
        local numGuildMembers, _ = GetNumGuildMembers();
        GuildRoster();
        for index=1, numGuildMembers do
            local memberName, _, _, _, _, _, _, _, connected, _, _, _, _, _, _ = GetGuildRosterInfo(index)
            if memberName == name then
                if connected then
                    isOnline = 1;
                else
                    isOnline = -1;
                end
            end
        end
    end

	if name ~= nil then	
		FlagRSP:PrintDebug("Online state for " .. name .. ": " .. isOnline);
	end

    -- -- Protocol Online Check
    if ( isOnline == 0 ) then
        isOnline = FlagRSPProtocol:CheckIsOnline( name );
	end

    -- WHO Online Check


    -- Buffer
	if ( name ~= nil ) then
		if self.onlineBufferList[name] == nil then
			self.onlineBufferList[name] = {};
		end

		self.onlineBufferList[name].isOnline = isOnline;
		self.onlineBufferList[name].timeStamp = GetTime();
	end

	return isOnline;
end

--[[

Friendlist:IsOnline(name)

-- returns true if player <name> is online.

]]--
function Friendlist:IsOnline(name)
	local isOnline = 0;

	if self.onlineBufferList[name] ~= nil and (GetTime()-self.onlineBufferList[name].timeStamp) <= 10  then
		isOnline = self.onlineBufferList[name].isOnline;
	else
		isOnline = self:checkOnline(name);
	end

	return isOnline;
end


--[[

Friendlist:GetOnlineInfo(name)

-- Get information if player name is online.

]]--
function Friendlist:GetOnlineInfo(name)
	local onlineInfo = "";
	local isOnline = self:IsOnline(name);

	if isOnline == 1 then
		onlineInfo = "|cff00c400" .. FRIENDLIST_LOCALE_OnlineLine .. "|r";
	elseif isOnline == -1 then
		onlineInfo = "|cff808080" .. FRIENDLIST_LOCALE_OfflineLine .. "|r";
	end

	return onlineInfo;
end


--[[

Friendlist:GetOnlineLocation(name)

-- Get information where player name currently is.

]]--
function Friendlist:GetOnlineLocation(name)
	local onlineLoc = "";

	for i=1, GetNumFriends() do
		local n, level, class, area, connected = GetFriendInfo(i);

		if n == name then
			if connected then
				onlineLoc = area;
			end
		end
	end

	if name == FlagRSP.pName then
		onlineLoc = GetRealZoneText();
	end

	return onlineLoc;
end


--[[

Friendlist:GetOnlineStatus(name)

-- Get status line for player's online status.

]]--
function Friendlist:GetOnlineStatus(name, playerID)
	local status = "";
    

	if self:GetType(name) == "char" then
        if strfind(name, "-") or strfind(name, " ") then
            return "";
        end
		local onlineInfo = self:GetOnlineInfo(name);
		if onlineInfo and onlineInfo ~= "" then
			local onlineLoc = self:GetOnlineLocation(name);
			if onlineLoc and onlineLoc ~= "" then
				status = "[" .. onlineInfo .. ", " .. onlineLoc .. "]";
			else
				status = "[" .. onlineInfo .. "]";
			end
		end
	end

	return status;
end

--[[

Friendlist:setButtonPosition(pos)

- Set position of Minimap button.
-- TODO: Change this to be handled by the FlagRSPConfigure methods.

]]--
function Friendlist:setButtonPosition(pos)
	-- Save position.
	Friendlist_Settings[Friendlist.realmName][Friendlist.playerName].buttonPosition = pos;

	-- Convert degree to radiant.
	local pos_rad = pos * 2 * (3.1415926536 / 360);
	local r = 81.500;

	-- Polar coordinates, yummy.
	local x = math.floor(r * math.cos(pos_rad)+0.5)-12;
	local y = math.floor(r * math.sin(pos_rad)+0.5)+12;

	FRIENDLISTMinimapButton:SetPoint("TOPLEFT", "Minimap", "CENTER", x, y);
end

--[[

Friendlist:UpdateMainFrame()

- Updates the settings (checkboxes) in the main frame of Friendlist.

]]--
function Friendlist:UpdateMainFrame()
end

-- Import all contacts from wow friendlist
-- TODO: Reimplement this.
function Friendlist:ImportFriendlist()
	Print_Chat("Import Friendlist...");
	for i=1, GetNumFriends(), 1 do
		local name, level, class, area, connected = GetFriendInfo(i);
		if (name) then
			if (name == "Unbekannt") then
			else
				if (FriendlistData.Friends[name] == nil) then
					local friend = { };
					friend.name = name;
                    friend.key = name;
					Friendlist:AddFriend(friend);
					-- Don't set friendstate...
				end -- if(not friendData...
			end -- if(name!=...
		end -- if(name)...
	end -- for i...

	-- Update UI
	FRIENDLIST_Update();
end -- ImportFriendlist()

------------------------------------------------------------------------------------------

-- Export friends to wow contactlist
-- TODO: Reimplement this.
function Friendlist:ExportFriendlist()
	Print_Chat("Export Friendlist...");
	
	if ((FriendlistData == nil) or
		(FriendlistData.Friends == nil)) then
		Print_Chat("FRIEND Data is empty - nothing to export");
	else
		-- iterate through frendlist
		for key, value in pairs(FriendlistData.Friends) do		
			local found = false;
			-- only add, if he doesn't exist allready
			for i=1, GetNumFriends(), 1 do
				local name, level, class, area, connected = GetFriendInfo(i);
				if(name == key) then
					found = true;
				end -- if(name...
			end -- for i=1
            -- Check if correct charname from own realm XXX dirty fix against crossrealms
            if ( strfind(key, "-") or strfind(key, " "))  then
                found = true;
            end
			if(found == false) then
				AddFriend(key);
			end -- if(found...
		end -- for key, value...
	end -- if((frienddata==nil)...

	-- Update UI
	FriendsList_Update();
end -- ExportFriendlist()


------------------------------------------------------------------------------------------

function addfriend_Msg(msg)
	if (msg == "") or (msg == nil) then
		Print_Chat("Friend List version: "..FRIENDLIST_VERSION_NUM);
		Print_Chat("type /list for more help");	
	else
		local idIndex = 1;
		local word = nil;
		local name = nil;
		local nachnameString = "";
		local noteString = "";
		for word in string.gmatch(msg, "%S+") do
			if (idIndex == 1) then
				name = word;
			elseif (idIndex == 2) then
				nachnameString = word;
			elseif (idIndex == 3) then
				noteString = word;
			elseif (idIndex > 3) then
				noteString = noteString .. " " .. word;
			end
			idIndex = idIndex + 1;
		end

		if ( (idIndex == 2) ) then
			nachnameString = nil;
			noteString = nil;
		end
		if (name ~= nil) then
			local friend = { };
			
			friend.name = name;
			friend.surname = nachnameString;
			friend.notes = noteString;
			
			self:AddFriend(friend);
		end
	end
end

-----------------------------------------------------------------------------------------
--[[

Below this comment block lies the accessor methods.  These methods are the only methods
allowed to directly interact with the friendData table.  Doing it this way allows me to
mark a line point that the friendData table should not appear above in order to keep the
code base clean.

]]--
-----------------------------------------------------------------------------------------

--[[

Friendlist:IsFriend(name)

- Check if player/object/guild is on Friendlist.

]]--
function Friendlist:IsFriend(name)
	--We can't check our Friendlist list if the variables aren't loaded or
	--our data structure isn't created yet
	if ( name == nil or name == "" ) then
		return false;
	end

	--If we get here, we are initialized and have valid data...check for FRIEND status	
	if( FriendlistData.Friends[name] == nil ) then
		return false;
	else
		return true;
	end
end

function Friendlist:GetKeyByIndex( index ) 
    -- print("D11: " .. index);
    local wantedKey = "";
    local playerID;
    
    if ( tonumber(index) ) then
        playerID = index;
    else
        playerID = tonumber(string.sub(index, 6, string.len(index)));
    end
    -- print("D12: " .. playerID);
   	
	if playerID ~= nil then
		for key, value in pairs(FriendlistData.Friends) do
            -- print("D13: " .. tostring(value.index));
			if ( value.index ~= nil and playerID == value.index ) then
				wantedKey = key;
                break;
			end
		end
	end
    
    return wantedKey;
end

--[[

Friendlist:GetName(playerID, first)

-- Accepts an index from the Friendlist and returns the character's name
   and Given name, but if first is specified, returns just the first name.

]]--
function Friendlist:GetName(playerID, first)
	local index = tonumber(string.sub(playerID, 6, string.len(playerID)));
	local unitName, firstName;
	
	if index ~= nil then
		for key, value in pairs(FriendlistData.Friends) do
			if ( value.index ~= nil and index == value.index ) then
				-- we found our entry, key is wanted name
				-- unitName = key;
				unitName = value.name;
				
				if ( value.altName ~= nil and value.altName ~= "" ) then
					-- Character has an alternate first name.
					firstName =  value.altName
				end
				
				if ( first == nil ) then
					-- Return both names, unitName and firstName.
					return unitName, firstName;
				else
					-- Return just the first name.
					return firstName;
				end
			end
		end
	end
end

--[[

Friendlist:GetAltName(name)

-- Returns an alternate name for <name>

]]--
function Friendlist:GetAltName(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].altName;
	end
    return "";
end

function Friendlist:SetName(key, name) 
	if ( FriendlistData.Friends[key] ~= nil ) then
		FriendlistData.Friends[key].altName = nil;
        FriendlistData.Friends[key].name = name; 
	end
end


--[[

Friendlist:SetAltName(name, altName)

-- Update and alternate name for <name>
@depraced
]]--
function Friendlist:SetAltName2(name, altName)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].altName = nil;
        FriendlistData.Friends[name].name = altName; 
	end
end

--[[

Friendlist:GetSurname(name)

- Get player names surname.

]]--
function Friendlist:GetSurname(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].surname;
	end
    return "";
end

function Friendlist:GetFullname(name)
    local fullname = Friendlist:GetFirstname(name) .. " " .. Friendlist:GetSurname(name);
    
    fullname = strtrim(fullname);
    
    return fullname;
	-- if ( FriendlistData.Friends[name] ~= nil ) then
		-- return FriendlistData.Friends[name].fullname;
	-- end
    -- return "";
end

function Friendlist:SetFullname(name, fullname)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].fullname = fullname;
	end
end

--[[

Friendlist:SetSurname(name, surname)

-- Update surname for entry <name>.

]]--
function Friendlist:SetSurname(name, surname)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].surname = surname;
	end
end

--[[

Friendlist:GetGuild(name)

-- Get player name's guild.

]]--
function Friendlist:GetGuild(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].guild;
	end
end

--[[

Friendlist:SetGuild(name, guild)

-- Update guild for entry <name>.

]]--
function Friendlist:SetGuild(name, guild)	
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].guild = guild;
	end
end

--[[

Friendlist:GetLevel(name)

-- Get player name's level.

]]--
function Friendlist:GetLevel(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		-- Although guilds will not be supported in normal view for future releases we have to capture their level for now.
		if ( self:GetType(name) == "char" ) then
			return FriendlistData.Friends[name].level;
		else
			return "";
		end
	else
		return "";
	end
end

--[[

Friendlist:SetLevel(name, level)

-- Update guild for entry <name>.

]]--
function Friendlist:SetLevel(name, level)
	if FriendlistData.Friends[name] ~= nil then
		FriendlistData.Friends[name].level = level;
	end
end

--[[

Friendlist:GetClass(name)

-- Get player name's class.

]]--
function Friendlist:GetClass(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].class;
	end
end

--[[

Friendlist:SetClass(name, class)

-- Update guild for entry <name>.

]]--
function Friendlist:SetClass(name, class)
	if FriendlistData.Friends[name] ~= nil then
		FriendlistData.Friends[name].class = class;
	end
end

--[[

Friendlist:GetRace(name)

-- Get player name's race.

]]--
function Friendlist:GetRace(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].race;
	end
end

--[[

Friendlist:SetRace(name, race)

-- Update race for entry <name>.

]]--
function Friendlist:SetRace(name, race)
	if FriendlistData.Friends[name] ~= nil then
		FriendlistData.Friends[name].race = race;
	end
end

--[[

Friendlist:GetRank(name)

-- Get player name's guild.

]]--
function Friendlist:GetRank(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].rank;
	end
end

--[[

Friendlist:SetRank(name, rank)

-- Update race for entry <name>.

]]--
function Friendlist:SetRank(name, rank)
	--FlagRSP:PrintDebug("Rank: " .. rank);
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].rank = rank;
	end
end

--[[

Friendlist:GetTitle(name)

-- Get player name's title.

]]--
function Friendlist:GetTitle( friendKey )
    if FriendlistHandler:ExistsEntry( friendKey ) then
        return FriendlistHandler:GetInfo( "title", friendKey );
    else
        return "";
    end
	-- if ( FriendlistData.Friends[name] ~= nil ) then
		-- return FriendlistData.Friends[name].title;
	-- end
end

--[[

Friendlist:SetTitle(name, title)

-- Update surname for entry <name>.

]]--
function Friendlist:SetTitle(name, title)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].title = title;
	end
end

--[[

Friendlist:SetFriendstate(name, friendstate)

-- Set the friendstate at entry name.

]]--
function Friendlist:SetFriendstate(name, friendstate)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].friendstate = friendstate;
	end
end

--[[

Friendlist:GetType(name)

- Get type for entry <name>.

]]--
function Friendlist:GetType(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].type;
	end
end

--[[

Friendlist:SetType(name, type)

-- Sets type to <type> at entry <name>.

]]--

function Friendlist:SetType(name, type)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].type = type;
	end
end

--[[

Friendlist:GetNotes(name)

- Get player names notes.

]]--

function Friendlist:GetNotes(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].notes;
	end
end


--[[

Friendlist:SetNotes(name, notes)

-- Sets the player notes to <notes> at <name>

]]--

function Friendlist:SetNotes(name, notes)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].notes = notes;
	end
end

--[[

Friendlist:GetFriendstate(name)

- Get player names friend status (currently only -10 for foe and 10 for friend).

]]--
function Friendlist:GetFriendstate(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].friendstate;
	end
end

--[[

Friendlist:SetFriendstate(name, friendstate)

-- Sets the friendstate for <name> to <friendstate>.

]]--

function Friendlist:SetFriendstate(name, friendstate)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].friendstate = friendstate;
	end
end

--[[

Friendlist:GetIndex(name)

- Returns index for entry <name>.

]]--
function Friendlist:GetIndex(name)
	if ( FriendlistData.Friends[name] ~= nil ) then
		return FriendlistData.Friends[name].index;
	end
end

--[[

Friendlist:SetIndex(name, index)

-- Sets the index at <name> to <index>.

]]--
function Friendlist:SetIndex(name, index)
	if ( FriendlistData.Friends[name] ~= nil ) then
		FriendlistData.Friends[name].index = index;
	end
end

--[[

Friendlist:GetFriends()

-- Returns the number of friends the player has in their Friendlist.
-- This AddOn, not the default Friends Lists.

]]--
function Friendlist:GetFriends()
	return FriendlistData.numFriends;
end

--[[

Friendlist:SetFriends(number)

-- Sets the number of friends in the Friendlist to <number>.

]]--
function Friendlist:SetFriends(number)
	FriendlistData.numFriends = number;
end

--[[

Friendlist:AddFriend(friend)

- Due to the fact that adding new data to the FriendlistData table requires editing of countless
  functions in order to add support for one new field, this function accepts a table containing
  relevant friend data and then stores it in the friendData table.

]]--
function Friendlist:AddFriend(friend)
	-- We need a name to store this information, if it's not there, exit.
	if ( friend.key == nil or friend.key == "" ) then
		return
	end
	
	-- Initialize all the data to empty strings if it's been passed as nil.
    if ( friend.name == nil ) then
        friend.name = friend.key;
    end
	if ( friend.notes == nil ) then
		friend.notes = "";
	end
	if ( friend.surname == nil ) then
		friend.surname = "";
	end
	if ( friend.title == nil ) then
		friend.title = "";
	end
	if ( friend.friendstate == nil ) then
		friend.friendstate = 0; -- Defaults to known.
	end
	if ( friend.newKey == nil ) then
		friend.newKey = ""; -- If this value were set, this would be the previous entry to edit.
	end
	-- if ( friend.altName == nil ) then
		-- friend.altName = "";
	-- end
	if ( friend.type == nil or friend.type == "" ) then
		friend.type = "char"; -- Friend is a character, not a guild.
	end
	
	-- Now, we add the data into the table.
	if ( friend.newKey == "" ) then
		-- We're adding a new entry, intialize it first.
		self:InitializeFriend(friend.key);
		
		-- Add in the basic information.
		self:SetSurname(friend.key, friend.surname);
		-- self:SetAltName(friend.key, friend.altName);
		self:SetName(friend.key, friend.name);
		self:SetNotes(friend.key, friend.notes);
		self:SetTitle(friend.key, friend.title);
		self:SetFriendstate(friend.key, friend.friendstate);
		self:SetType(friend.key, friend.type);
		
		-- Initialize these to empty values and let the normal update code handle them.
		self:SetClass(friend.key, "");
		self:SetLevel(friend.key, 0);
		self:SetRace(friend.key, "");
		self:SetGuild(friend.key, "");
		
		-- Print out successful adding of friend.
		FlagRSPStatic:PrintM(string.gsub(FRIENDLIST_LOCALE_AddFriendMsg, "%%n", friend.key));
	else
		-- -- Editing an old entry.  Shouldn't I have an edit function do this?
		-- -- TODO Hey, what do you know, I do.  Keep this until I've converted all edits to use the edit function.
		-- self:SetSurname(friend.key, friend.surname);
		-- self:SetAltName(friend.key, friend.altName);
		-- self:SetNotes(friend.key, friend.notes);
		-- self:SetTitle(friend.key, friend.title);
		-- self:SetFriendstate(friend.key, friend.title);
		
		-- if ( friend.newKey ~= "" and friend.newKey ~= nil and friend.newKey ~= friend.key ) then
			-- self:MoveFriend(friend.newKey, friend.key);
		-- end
	end
	
	-- XXX Calling this for safety for now, when complete, look through and see if this can be fixed up.
	FRIENDLIST_Update();
end

--[[

Friendlist:InitializeData()

-- Creates a new FriendlistData structure.
-- Warning: This doesn't care if FriendData already exists.

]]--
function Friendlist:InitializeData()
    FriendlistHandler:Init()
	-- FriendlistData = { };
	-- FriendlistData.Friends = { };
	
	-- self:SetFriends(0);
end

--[[

Friendlist:InitializeFriend(name)

-- Initializes entry in friendData[realmName][playerName].FRIEND[name].
-- If the entry already exists, return immediately, otherwise initialize the entry.

]]--
function Friendlist:InitializeFriend(name)
	if ( name == nil or name == "" ) then
		return;
	end
	
	-- Test to make sure the friend doesn't exist.
	if ( FriendlistData.Friends[name] == nil ) then
		FriendlistData.Friends[name] = { };
		
		-- This data is necessary for the FauxScrollFrame to render the Friendlist correctly.
		local newIndex = self:GetFriends() + 1;
		self:SetIndex(name, newIndex);
		self:SetFriends(newIndex);
	end
end

--[[

Friendlist:MoveFriend(new, old)

-- Copies the friend entry at old to the friend entry at new and deletes the entry at old.

]]--
function Friendlist:MoveFriend(new, old)
    FriendlistHandler:ChangeKey(old, new);
end

--[[

Friendlist:EditFriend(friend)

-- Parameters:
--   friend - Table containing friend data.
-- Edits the friend data at name to contain the values in friend.
-- If friend.newName is set, MoveFriend will be called.

]]--
function Friendlist:EditFriend(friend)
	-- The previous name or the character's current name needs to exist.
	if ( friend.key == nil or friend.key == "" ) then
		return;
	end
	
	-- Initialize all the data to empty strings if it's been passed as nil.
	if ( friend.notes == nil ) then
		friend.notes = "";
	end
	if ( friend.surname == nil ) then
		friend.surname = "";
	end
	if ( friend.title == nil ) then
		friend.title = "";
	end
	if ( friend.friendstate == nil ) then
		friend.friendstate = 0; -- Defaults to known.
	end
	-- if ( friend.altName == nil ) then
		-- friend.altName = "";
	-- end#
	
    if friend.type == "char" then
        self:SetName(friend.key, friend.name);
        -- self:SetAltName(friend.key, friend.altName);
        self:SetSurname(friend.key, friend.surname);
        self:SetTitle(friend.key, friend.title);
        self:SetFriendstate(friend.key, friend.friendstate);
        self:SetNotes(friend.key, friend.notes);
    elseif friend.type == "guild" then
        self:SetName(friend.key, friend.key);
        -- self:SetAltName(friend.key, friend.altName);
        -- self:SetSurname(friend.key, friend.surname);
        -- self:SetTitle(friend.key, friend.title);
        self:SetFriendstate(friend.key, friend.friendstate);
        self:SetNotes(friend.key, friend.notes);
    end
	
	-- Test to see if a new name has been passed.
	if ( friend.newKey ~= nil and friend.newKey ~= "" and friend.newKey ~= friend.key ) then
		-- Move the friend up, to the east side.
		self:MoveFriend(friend.newKey, friend.key);
	end
end

--[[

Friendlist:DeleteFriend(name)

-- Deletes entry at <name> and decrements the numFRIEND and reorders the indices.

]]--
function Friendlist:DeleteFriend(name)
	if ( name == nil or name == "" ) then
		return;
	end
	
	-- Perserver the old index.
	local oldIndex = self:GetIndex(name);
	
	-- Break up with the old friend.
	FriendlistData.Friends[name] = nil;
	self:SetFriends(self:GetFriends() - 1);
	FlagRSPStatic:PrintM(string.gsub(FRIENDLIST_LOCALE_DelFriendMsg, "%%n", name));
	
	for key in pairs(FriendlistData.Friends) do
		if ( self:GetIndex(key) > oldIndex ) then
			self:SetIndex(key, self:GetIndex(key) - 1);
		end
	end
end

--[[

Friendlist:GetFriend(index)

-- Returns a table containing friend info retrieved from the Friendlist:Get*() functions.
-- I'd rather do it that way instead of just returning the table at Friends[name] in case
-- I change the keys to something stupid in a fit of madness.

]]--
function Friendlist:GetFriend(index)
	local friend = { };
	
	for key, value in pairs(FriendlistData.Friends) do
		if ( self:GetIndex(key) == index ) then
			friend.key = key;
			friend.surname = self:GetSurname(key);
			friend.altName = self:GetAltName(key)
			friend.class = self:GetClass(key);
			friend.guild = self:GetGuild(key);
			friend.level = self:GetLevel(key);
			friend.friendstate = self:GetFriendstate(key);
			friend.notes = self:GetNotes(key);
		end
	end
	
	return friend;
end

--[[

Friendlist:GetFriendsTable()

-- This returns the table that stores all the friend information.
-- This is used for the sorting functions in the Friendlist.

]]--
function Friendlist:GetFriendsTable()
	if ( FriendlistData.Friends ~= nil ) then
		return FriendlistData.Friends;
	end
end

--[[

Friendlist:ResetFriendlist()

-- Empties the Friends element and set the amount of Friends to 0.

]]--

function Friendlist:ResetFriendlist()
	FriendlistData.Friends = { };
	FriendlistData.SetFriends(0);
end

--[[

Friendlist:ResetFriends()

-- Clears out the stored data stored for the the friends, but keeps their entries.

]]--

function Friendlist:ResetFriends()
	for key in pairs(FriendlistData.Friends) do
		self:SetNotes(key, "");
		self:SetClass(key, "");
		self:SetSurname(key, "");
		self:SetGuild(key, "");
		self:SetLevel(key, 0);
		self:SetFriendstate(key, 0);
	end
end

--[[

- Friendlist:PrintFriendlist()

-- Prints out the contents of the Friendlist to the chat frame.

]]--

function Friendlist:PrintFriendlist()
	for key, value in pairs(FriendlistData.Friends) do
		Print_Chat("Name: " .. key);
		Print_Chat("   Surname: " .. self:GetSurname(key));
		Print_Chat("   Given Name: " .. self:GetAltName(key));
		Print_Chat("   Class: " .. self:GetClass(key));
		Print_Chat("   Guild: " .. self:GetGuild(key));
		Print_Chat("   Notes: " .. self:GetNotes(key));
		Print_Chat("   Level: " .. self:GetLevel(key));
	end
end

function Friendlist:hasNotes( name )
    if Friendlist == nil then
        return false;
    end
    if not Friendlist:IsFriend(name) then
        return false;
    end
    return Friendlist:GetNotes(name) ~= nil and strtrim(Friendlist:GetNotes(name)) ~= "";
end

function Friendlist:GetFirstname( key )
    local altName, name;
    local firstname = "";
    
    altName   = FriendlistHandler:GetInfo( "altName", key );
    name      = FriendlistHandler:GetInfo( "name", key );
    
    firstname = firstname .. " " .. altName .. " " .. name;    
    firstname = strtrim(firstname);
    
    return firstname;
end

function Friendlist:GetSurname( key )
    local lastName;
    
    lastName  = FriendlistHandler:GetInfo( "surname", key );
    
    lastName = strtrim(lastName);
    
    return lastName;
end

function Friendlist:GetFriendKey( index )
    return FriendlistHandler:GetKeyByIndex( index );
end


--[[ Returns the index of a friend in the list
-- local string = FriendlistHandler:GetKeyByIndex( playerID )
-- Returns the friend index
-- Params:
--      playerID    can be in the format "index#" with # as a number or directly as a number #
--]]
function FriendlistHandler:GetKeyByIndex( playerID ) -- TODO belongs to friendlist.lua (abstract from content of the friend index)
	if ( playerID == nil or playerID == "" ) then
        return "";
	end
	
	local indexKey = "";

    if ( string.sub(playerID, 1, 5) == "index" ) then
        local id = tonumber(string.sub(playerID, 6, string.len(playerID)));
        for k, v in pairs(FriendlistData.Friends) do
            if ( v.index ~= nil and id == v.index ) then
                indexKey = k;
                break;
            end
        end
    elseif ( type(playerID) == "number" ) then
        local id = playerID;
        for k, v in pairs(FriendlistData.Friends) do
            if ( v.index ~= nil and id == v.index ) then
                indexKey = k;
                break;
            end
        end
    end

    return indexKey;
end


--[[ Returns statistic information on the friendlist
-- local numFriends, typeChar, typeNPC, typeGuild, stateFriend, stateNeutral, stateEnemy = FriendlistHandler:GetStats( show )
    Returns the number of entries matching a specific type or state
    - show: if true, this method prints the statistic directly
    - Returns integer values of
        numFriends overall entries n your friendlist
        number of entries with the type character, npc or guild in your friendlist
        number of entries with the state friend, neutral and enemy
]]--
function FriendlistHandler:GetStats( show ) -- TODO belongs to friendlist, make on more general for this
    local numFriends, typeChar, typeNPC, typeGuild = 0, 0, 0, 0;
    local stateFriend, stateNeutral, stateEnemy = 0, 0, 0;

    for entry, t in pairs(FriendlistData.Friends) do
        if ( t.type == "char" ) then
            typeChar = typeChar + 1;
        elseif ( t.type == "npc" ) then
            typeNPC = typeNPC + 1;
        elseif ( t.type == "npc" ) then
            typeGuild = typeGuild + 1;
        end

        if ( t.friendstate > 0 ) then
            stateFriend = stateFriend + 1;
        elseif ( t.friendstate == 0 ) then
            stateNeutral = stateNeutral + 1;
        elseif ( t.friendstate < 0 ) then
            stateEnemy = stateEnemy + 1;
        end

        numFriends = numFriends + 1;
    end

    if ( show ) then
        print("<FlagRSP Friendlist> Friendlist statistics:");
        print("<FlagRSP Friendlist>    Friends: " .. numFriends);
        print("<FlagRSP Friendlist>       Chars: " .. typeChar);
        print("<FlagRSP Friendlist>       NPC: " .. typeNPC);
        print("<FlagRSP Friendlist>       Guilds: " .. typeGuild);
        print("<FlagRSP Friendlist>    Friendstate:");
        print("<FlagRSP Friendlist>       Friend: " .. stateFriend);
        print("<FlagRSP Friendlist>       Neutral: " .. stateNeutral);
        print("<FlagRSP Friendlist>       Enemy: " .. stateEnemy);
    end

    return numFriends, typeChar, typeNPC, typeGuild, stateFriend, stateNeutral, stateEnemy;
end

-- function Friendlist:InitVar()

-- end

-- function Friendlist:Init()

-- end

--[[ Fixes the table for the last version of FriendlistHandler --]]
function Friendlist:FixSavedVariables() -- TODO belongs to Friendlist
    if FriendlistData == nil then
        return
    end

    local t = FriendlistData.Friends;

    if FriendlistData.version == nil then -- to 20130317
        for i, v in pairs(t) do
            if v.type == "char" then
                v.name = v.altName .. " (" .. v.name..")";
                v.name = strtrim(v.name);
            end

            v.altName = nil;
        end
    end

    FriendlistData.version = FriendlistHandler:GetVersion();
end
-- /script local t=FriendlistData.Friends;for i,v in pairs(t) do v.name = v.altName .. " " .. v.name;v.altName=nil; end



        -- -- TODO Change this to be more general -> this belongs into Friendlist.lua
        -- if ( field == "type" or field == "class" or field == "race"
                -- or field == "surname" or field == "name" or field == "altName" or field == "title"
                -- or field == "guild" or field == "notes" or field == "rank"
                -- ) then
            -- entry[field] = value or "";
        -- elseif ( field == "level" or field == "friendstate" ) then
            -- entry[field] = tonumber(value) or 0;
        -- end